@@ -1,77 +1,51 @@
-/*  This file is part of the Vc library.
+/*  This file is part of the Vc library. {{{
+Copyright © 2009-2015 Matthias Kretz <kretz@kde.org>
 
-    Copyright (C) 2009-2012 Matthias Kretz <kretz@kde.org>
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+    * Redistributions of source code must retain the above copyright
+      notice, this list of conditions and the following disclaimer.
+    * Redistributions in binary form must reproduce the above copyright
+      notice, this list of conditions and the following disclaimer in the
+      documentation and/or other materials provided with the distribution.
+    * Neither the names of contributing organizations nor the
+      names of its contributors may be used to endorse or promote products
+      derived from this software without specific prior written permission.
 
-    Vc is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as
-    published by the Free Software Foundation, either version 3 of
-    the License, or (at your option) any later version.
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-    Vc is distributed in the hope that it will be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+}}}*/
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Vc.  If not, see <http://www.gnu.org/licenses/>.
-
-*/
-
-#ifndef VC_COMMON_MEMORYBASE_H
-#define VC_COMMON_MEMORYBASE_H
+#ifndef VC_COMMON_MEMORYBASE_H_
+#define VC_COMMON_MEMORYBASE_H_
 
 #include <assert.h>
+#include <type_traits>
+#include <iterator>
 #include "macros.h"
 
-/*OUTER_NAMESPACE_BEGIN*/
-namespace Vc
+namespace Vc_VERSIONED_NAMESPACE
+{
+namespace Common
 {
 
-#if __cplusplus >= 201103 || defined(VC_MSVC)
-#define VC_DECLTYPE(T1, op, T2) decltype(T1() op T2())
-#elif defined(VC_OPEN64) || (defined(VC_GCC) && VC_GCC < 0x40300)
-#define VC_DECLTYPE(T1, op, T2) T1
-#else
-namespace
-{
-    struct one { char x; };
-    struct two { one x, y; };
-    template<typename T1, typename T2> struct DecltypeHelper
-    {
-        static one test(const T1 &) { return one(); }
-        static two test(const T2 &) { return two(); }
-        //static void test(...) {}
-    };
-    template<typename T1> struct DecltypeHelper<T1, T1>
-    {
-        static one test(const T1 &) { return one(); }
-        //static void test(...) {}
-    };
-    template<typename T1, typename T2, size_t ToSelect> struct Decltype { typedef T1 Value; };
-    template<typename T1, typename T2> struct Decltype<T1, T2, sizeof(one)> { typedef T1 Value; };
-    template<typename T1, typename T2> struct Decltype<T1, T2, sizeof(two)> { typedef T2 Value; };
-#ifdef VC_CLANG
-    // this special case is only necessary to silence a warning (which is rather a note that clang
-    // did the expected optimization):
-    // warning: variable 'SOME_PTR' is not needed and will not be emitted [-Wunneeded-internal-declaration]
-    // Then again, I don't remember why the SOME_PTR hack was necessary in the first place - some
-    // strange compiler quirk...
-#define VC_DECLTYPE(T1, op, T2) typename Decltype<T1, T2, sizeof(DecltypeHelper<T1, T2>::test(T1() op T2()))>::Value
-#else
-    static const void *SOME_PTR;
-#define VC_DECLTYPE(T1, op, T2) typename Decltype<T1, T2, sizeof(DecltypeHelper<T1, T2>::test(*static_cast<const T1*>(SOME_PTR) op *static_cast<const T2*>(SOME_PTR)))>::Value
-#endif
-} // anonymous namespace
-#endif
-
-#define VC_MEM_OPERATOR_EQ(op) \
+#define Vc_MEM_OPERATOR_EQ(op) \
         template<typename T> \
-        Vc_ALWAYS_INLINE VectorPointerHelper<V, A> &operator op##=(const T &x) { \
-            const V result = V(m_ptr, Internal::FlagObject<A>::the()) op x; \
-            result.store(m_ptr, Internal::FlagObject<A>::the()); \
+        Vc_ALWAYS_INLINE enable_if_mutable<T, MemoryVector &> operator op##=(const T &x) { \
+            const V v = value() op x; \
+            v.store(&m_data[0], Flags()); \
             return *this; \
         }
-
+/*dox{{{*/
 /**
  * Helper class for the Memory::vector(size_t) class of functions.
  *
@@ -79,71 +53,149 @@ namespace
  * Memory API.
  *
  * \headerfile memorybase.h <Vc/Memory>
- */
-template<typename V, typename A> class VectorPointerHelperConst
+ *//*}}}*/
+template<typename _V, typename Flags> class MemoryVector/*{{{*/
 {
+    typedef typename std::remove_cv<_V>::type V;
+
+    template<typename T, typename R> using enable_if_mutable =
+        typename std::enable_if<std::is_same<T, T>::value && !std::is_const<_V>::value, R>::type;
+
     typedef typename V::EntryType EntryType;
     typedef typename V::Mask Mask;
-    public:
-        const EntryType *const m_ptr;
 
-        explicit VectorPointerHelperConst(const EntryType *ptr) : m_ptr(ptr) {}
+    EntryType m_data[V::Size];
+public:
+        // It is important that neither initialization nor cleanup is done as MemoryVector aliases
+        // other memory
+        Vc_ALWAYS_INLINE MemoryVector() {}
+
+        // disable copies because this type is supposed to alias the data in a Memory object,
+        // nothing else
+        MemoryVector(const MemoryVector &) = delete;
+        MemoryVector(MemoryVector &&) = delete;
+        // Do not disable MemoryVector &operator=(const MemoryVector &) = delete; because it is
+        // covered nicely by the operator= below.
+
+        //! \internal
+        Vc_ALWAYS_INLINE Vc_PURE V value() const { return V(&m_data[0], Flags()); }
 
         /**
          * Cast to \p V operator.
          *
          * This function allows to assign this object to any object of type \p V.
          */
-        Vc_ALWAYS_INLINE Vc_PURE operator const V() const { return V(m_ptr, Internal::FlagObject<A>::the()); }
-};
-
-/**
- * Helper class for the Memory::vector(size_t) class of functions.
- *
- * You will never need to directly make use of this class. It is an implementation detail of the
- * Memory API.
- *
- * \headerfile memorybase.h <Vc/Memory>
- */
-template<typename V, typename A> class VectorPointerHelper
-{
-    typedef typename V::EntryType EntryType;
-    typedef typename V::Mask Mask;
-    public:
-        EntryType *const m_ptr;
-
-        explicit VectorPointerHelper(EntryType *ptr) : m_ptr(ptr) {}
-
-        /**
-         * Cast to \p V operator.
-         *
-         * This function allows to assign this object to any object of type \p V.
-         */
-        Vc_ALWAYS_INLINE Vc_PURE operator const V() const { return V(m_ptr, Internal::FlagObject<A>::the()); }
+        Vc_ALWAYS_INLINE Vc_PURE operator V() const { return value(); }
 
         template<typename T>
-        Vc_ALWAYS_INLINE VectorPointerHelper &operator=(const T &x) {
+        Vc_ALWAYS_INLINE enable_if_mutable<T, MemoryVector &> operator=(const T &x) {
             V v;
             v = x;
-            v.store(m_ptr, Internal::FlagObject<A>::the());
+            v.store(&m_data[0], Flags());
             return *this;
         }
 
-        VC_ALL_BINARY(VC_MEM_OPERATOR_EQ)
-        VC_ALL_ARITHMETICS(VC_MEM_OPERATOR_EQ)
+        Vc_ALL_BINARY(Vc_MEM_OPERATOR_EQ);
+        Vc_ALL_ARITHMETICS(Vc_MEM_OPERATOR_EQ);
 };
-#undef VC_MEM_OPERATOR_EQ
 
-#define VC_VPH_OPERATOR(op) \
-template<typename V1, typename A1, typename V2, typename A2> \
-VC_DECLTYPE(V1, op, V2) operator op(const VectorPointerHelper<V1, A1> &x, const VectorPointerHelper<V2, A2> &y) { \
-    return V1(x.m_ptr, Internal::FlagObject<A1>::the()) op V2(y.m_ptr, Internal::FlagObject<A2>::the()); \
+template<typename _V, typename Flags> class MemoryVectorIterator
+{
+    typedef typename std::remove_cv<_V>::type V;
+
+    template<typename T, typename R> using enable_if_mutable =
+        typename std::enable_if<std::is_same<T, T>::value && !std::is_const<_V>::value, R>::type;
+
+    using iterator_traits = std::iterator_traits<MemoryVector<_V, Flags> *>;
+
+    MemoryVector<_V, Flags> *d;
+public:
+    typedef typename iterator_traits::difference_type difference_type;
+    typedef typename iterator_traits::value_type value_type;
+    typedef typename iterator_traits::pointer pointer;
+    typedef typename iterator_traits::reference reference;
+    typedef typename iterator_traits::iterator_category iterator_category;
+
+    constexpr MemoryVectorIterator(MemoryVector<_V, Flags> *dd) : d(dd) {}
+    constexpr MemoryVectorIterator(const MemoryVectorIterator &) = default;
+    constexpr MemoryVectorIterator(MemoryVectorIterator &&) = default;
+    Vc_ALWAYS_INLINE MemoryVectorIterator &operator=(const MemoryVectorIterator &) = default;
+
+    Vc_ALWAYS_INLINE void *orderBy() const { return d; }
+
+    Vc_ALWAYS_INLINE difference_type operator-(const MemoryVectorIterator &rhs) const { return d - rhs.d; }
+    Vc_ALWAYS_INLINE reference operator[](size_t i) const { return d[i]; }
+    Vc_ALWAYS_INLINE reference operator*() const { return *d; }
+    Vc_ALWAYS_INLINE pointer operator->() const { return d; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator &operator++() { ++d; return *this; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator operator++(int) { MemoryVectorIterator r(*this); ++d; return r; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator &operator--() { --d; return *this; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator operator--(int) { MemoryVectorIterator r(*this); --d; return r; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator &operator+=(size_t n) { d += n; return *this; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator &operator-=(size_t n) { d -= n; return *this; }
+    Vc_ALWAYS_INLINE MemoryVectorIterator operator+(size_t n) const { return MemoryVectorIterator(d + n); }
+    Vc_ALWAYS_INLINE MemoryVectorIterator operator-(size_t n) const { return MemoryVectorIterator(d - n); }
+};
+
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator==(const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() == r.orderBy();
 }
-VC_ALL_ARITHMETICS(VC_VPH_OPERATOR)
-VC_ALL_BINARY     (VC_VPH_OPERATOR)
-VC_ALL_COMPARES   (VC_VPH_OPERATOR)
-#undef VC_VPH_OPERATOR
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator!=(const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() != r.orderBy();
+}
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator>=(const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() >= r.orderBy();
+}
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator<=(const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() <= r.orderBy();
+}
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator> (const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() >  r.orderBy();
+}
+template<typename V, typename FlagsL, typename FlagsR>
+Vc_ALWAYS_INLINE bool operator< (const MemoryVectorIterator<V, FlagsL> &l, const MemoryVectorIterator<V, FlagsR> &r)
+{
+    return l.orderBy() <  r.orderBy();
+}
+/*}}}*/
+#undef Vc_MEM_OPERATOR_EQ
 
+#define Vc_VPH_OPERATOR(op)                                                              \
+    template <typename V1, typename Flags1, typename V2, typename Flags2>                \
+    decltype(std::declval<V1>() op std::declval<V2>()) operator op(                      \
+        const MemoryVector<V1, Flags1> &x, const MemoryVector<V2, Flags2> &y)            \
+    {                                                                                    \
+        return x.value() op y.value();                                                   \
+    }
+Vc_ALL_ARITHMETICS(Vc_VPH_OPERATOR);
+Vc_ALL_BINARY     (Vc_VPH_OPERATOR);
+Vc_ALL_COMPARES   (Vc_VPH_OPERATOR);
+#undef Vc_VPH_OPERATOR
+
+template<typename V, typename Parent, typename Flags = Prefetch<>> class MemoryRange/*{{{*/
+{
+    Parent *m_parent;
+    size_t m_first;
+    size_t m_last;
+
+public:
+    MemoryRange(Parent *p, size_t firstIndex, size_t lastIndex)
+        : m_parent(p), m_first(firstIndex), m_last(lastIndex)
+    {}
+
+    MemoryVectorIterator<V, Flags> begin() const { return &m_parent->vector(m_first   , Flags()); }
+    MemoryVectorIterator<V, Flags> end() const   { return &m_parent->vector(m_last + 1, Flags()); }
+};/*}}}*/
 template<typename V, typename Parent, int Dimension, typename RowMemory> class MemoryDimensionBase;
 template<typename V, typename Parent, typename RowMemory> class MemoryDimensionBase<V, Parent, 1, RowMemory> // {{{1
 {
@@ -170,6 +222,7 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
         /// Const overload of the above function.
         Vc_ALWAYS_INLINE Vc_PURE const EntryType scalar(size_t i) const { return entries()[i]; }
 
+#ifdef DOXYGEN
         /**
          * Cast operator to the scalar type. This allows to use the object very much like a standard
          * C array.
@@ -177,17 +230,52 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
         Vc_ALWAYS_INLINE Vc_PURE operator       EntryType*()       { return entries(); }
         /// Const overload of the above function.
         Vc_ALWAYS_INLINE Vc_PURE operator const EntryType*() const { return entries(); }
+#else
+        // The above conversion operator allows implicit conversion to bool. To prohibit this
+        // conversion we use SFINAE to allow only conversion to EntryType* and void*.
+        template <typename T,
+                  typename std::enable_if<
+                      std::is_same<typename std::remove_const<T>::type, EntryType *>::value ||
+                          std::is_same<typename std::remove_const<T>::type, void *>::value,
+                      int>::type = 0>
+        Vc_ALWAYS_INLINE Vc_PURE operator T()
+        {
+            return entries();
+        }
+        template <typename T,
+                  typename std::enable_if<std::is_same<T, const EntryType *>::value ||
+                                              std::is_same<T, const void *>::value,
+                                          int>::type = 0>
+        Vc_ALWAYS_INLINE Vc_PURE operator T() const
+        {
+            return entries();
+        }
+#endif
+
+        /**
+         *
+         */
+        template<typename Flags>
+        Vc_ALWAYS_INLINE MemoryRange<V, Parent, Flags> range(size_t firstIndex, size_t lastIndex, Flags) {
+            return MemoryRange<V, Parent, Flags>(p(), firstIndex, lastIndex);
+        }
+        Vc_ALWAYS_INLINE MemoryRange<V, Parent> range(size_t firstIndex, size_t lastIndex) {
+            return MemoryRange<V, Parent>(p(), firstIndex, lastIndex);
+        }
+        template<typename Flags>
+        Vc_ALWAYS_INLINE MemoryRange<const V, Parent, Flags> range(size_t firstIndex, size_t lastIndex, Flags) const {
+            return MemoryRange<const V, Parent, Flags>(p(), firstIndex, lastIndex);
+        }
+        Vc_ALWAYS_INLINE MemoryRange<const V, Parent> range(size_t firstIndex, size_t lastIndex) const {
+            return MemoryRange<const V, Parent>(p(), firstIndex, lastIndex);
+        }
 
-        // omit operator[] because the EntryType* cast operator suffices, for dox it makes sense to
-        // show it, though because it helps API discoverability.
-#ifdef DOXYGEN
         /**
          * Returns the \p i-th scalar value in the memory.
          */
-        inline EntryType &operator[](size_t i);
+        Vc_ALWAYS_INLINE EntryType &operator[](size_t i) { return entries()[i]; }
         /// Const overload of the above function.
-        inline const EntryType &operator[](size_t i) const;
-#endif
+        Vc_ALWAYS_INLINE const EntryType &operator[](size_t i) const { return entries()[i]; }
 
         /**
          * Uses a vector gather to combine the entries at the indexes in \p i into the returned
@@ -216,7 +304,7 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
          */
         typedef typename V::EntryType EntryType;
 
-        static _VC_CONSTEXPR size_t rowCount() { return Parent::RowCount; }
+        static constexpr size_t rowCount() { return Parent::RowCount; }
 
         /**
          * Returns a pointer to the start of the allocated memory.
@@ -236,11 +324,19 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
          * Returns the \p i-th row in the memory.
          */
         Vc_ALWAYS_INLINE Vc_PURE RowMemory &operator[](size_t i) {
+#ifdef Vc_RECURSIVE_MEMORY
+            return p()->m_mem[i];
+#else
             return RowMemory::fromRawData(entries(i));
+#endif
         }
         /// Const overload of the above function.
         Vc_ALWAYS_INLINE Vc_PURE const RowMemory &operator[](size_t i) const {
+#ifdef Vc_RECURSIVE_MEMORY
+            return p()->m_mem[i];
+#else
             return RowMemory::fromRawData(const_cast<EntryType *>(entries(i)));
+#endif
         }
 
         /**
@@ -251,7 +347,7 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
         Vc_ALWAYS_INLINE Vc_PURE size_t rowsCount() const { return p()->rowsCount(); }
 };
 
-//{{{1
+//dox{{{1
 /**
  * \headerfile memorybase.h <Vc/Memory>
  *
@@ -264,6 +360,10 @@ template<typename V, typename Parent, typename RowMemory> class MemoryDimensionB
  */
 template<typename V, typename Parent, int Dimension, typename RowMemory> class MemoryBase : public MemoryDimensionBase<V, Parent, Dimension, RowMemory> //{{{1
 {
+    static_assert((V::size() * sizeof(typename V::EntryType)) % V::MemoryAlignment == 0,
+                  "Vc::Memory can only be used for data-parallel types storing a number "
+                  "of values that's a multiple of the memory alignment.");
+
     private:
         Parent *p() { return static_cast<Parent *>(this); }
         const Parent *p() const { return static_cast<const Parent *>(this); }
@@ -287,6 +387,24 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
         using MemoryDimensionBase<V, Parent, Dimension, RowMemory>::entries;
         using MemoryDimensionBase<V, Parent, Dimension, RowMemory>::scalar;
 
+        /**
+         * Return a (vectorized) iterator to the start of this memory object.
+         */
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE MemoryVectorIterator<      V, Flags> begin(Flags flags = Flags())       { return &firstVector(flags); }
+        //! const overload of the above
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE MemoryVectorIterator<const V, Flags> begin(Flags flags = Flags()) const { return &firstVector(flags); }
+
+        /**
+         * Return a (vectorized) iterator to the end of this memory object.
+         */
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE MemoryVectorIterator<      V, Flags>   end(Flags flags = Flags())       { return &lastVector(flags) + 1; }
+        //! const overload of the above
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE MemoryVectorIterator<const V, Flags>   end(Flags flags = Flags()) const { return &lastVector(flags) + 1; }
+
         /**
          * \param i Selects the offset, where the vector should be read.
          *
@@ -307,8 +425,9 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          * access memory at fixed strides. If access to known offsets from the aligned vectors is
          * needed the vector(size_t, int) function can be used.
          */
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, AlignedFlag> vector(size_t i) {
-            return VectorPointerHelper<V, AlignedFlag>(&entries()[i * V::Size]);
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE typename std::enable_if<!std::is_convertible<Flags, int>::value, MemoryVector<V, Flags>>::type &vector(size_t i, Flags = Flags()) {
+            return *new(&entries()[i * V::Size]) MemoryVector<V, Flags>;
         }
         /** \brief Const overload of the above function
          *
@@ -316,8 +435,9 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          *
          * \return a smart object to wrap the \p i-th vector in the memory.
          */
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, AlignedFlag> vector(size_t i) const {
-            return VectorPointerHelperConst<V, AlignedFlag>(&entries()[i * V::Size]);
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE typename std::enable_if<!std::is_convertible<Flags, int>::value, MemoryVector<const V, Flags>>::type &vector(size_t i, Flags = Flags()) const {
+            return *new(const_cast<EntryType *>(&entries()[i * V::Size])) MemoryVector<const V, Flags>;
         }
 
         /**
@@ -335,12 +455,14 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          * \param i      Specifies the scalar entry from where the vector will be loaded/stored. I.e. the
          * values scalar(i), scalar(i + 1), ..., scalar(i + V::Size - 1) will be read/overwritten.
          *
-         * \param align  You must take care to determine whether an unaligned load/store is
-         * required. Per default an aligned load/store is used. If \p i is not a multiple of \c V::Size
-         * you must pass Vc::Unaligned here.
+         * \param flags  You must take care to determine whether an unaligned load/store is
+         * required. Per default an unaligned load/store is used. If \p i is a multiple of \c V::Size
+         * you may want to pass Vc::Aligned here.
          */
-#ifdef DOXYGEN
-        template<typename A> inline VectorPointerHelper<V, A> vectorAt(size_t i, A align = Vc::Aligned);
+        template<typename Flags = UnalignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<V, Flags> &vectorAt(size_t i, Flags flags = Flags()) {
+            return *new(&entries()[i]) MemoryVector<V, Flags>;
+        }
         /** \brief Const overload of the above function
          *
          * \return a smart object to wrap the vector starting from the \p i-th scalar entry in the memory.
@@ -348,28 +470,14 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          * \param i      Specifies the scalar entry from where the vector will be loaded/stored. I.e. the
          * values scalar(i), scalar(i + 1), ..., scalar(i + V::Size - 1) will be read/overwritten.
          *
-         * \param align  You must take care to determine whether an unaligned load/store is
-         * required. Per default an aligned load/store is used. If \p i is not a multiple of \c V::Size
-         * you must pass Vc::Unaligned here.
+         * \param flags  You must take care to determine whether an unaligned load/store is
+         * required. Per default an unaligned load/store is used. If \p i is a multiple of \c V::Size
+         * you may want to pass Vc::Aligned here.
          */
-        template<typename A> inline const VectorPointerHelperConst<V, A> vectorAt(size_t i, A align = Vc::Aligned) const;
-#else
-        template<typename A>
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, A> vectorAt(size_t i, A) {
-            return VectorPointerHelper<V, A>(&entries()[i]);
+        template<typename Flags = UnalignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<const V, Flags> &vectorAt(size_t i, Flags flags = Flags()) const {
+            return *new(const_cast<EntryType *>(&entries()[i])) MemoryVector<const V, Flags>;
         }
-        template<typename A>
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, A> vectorAt(size_t i, A) const {
-            return VectorPointerHelperConst<V, A>(&entries()[i]);
-        }
-
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, AlignedFlag> vectorAt(size_t i) {
-            return VectorPointerHelper<V, AlignedFlag>(&entries()[i]);
-        }
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, AlignedFlag> vectorAt(size_t i) const {
-            return VectorPointerHelperConst<V, AlignedFlag>(&entries()[i]);
-        }
-#endif
 
         /**
          * \return a smart object to wrap the \p i-th vector + \p shift in the memory.
@@ -398,12 +506,24 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          * mem.vector(0, i) += 1;
          * \endcode
          */
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, UnalignedFlag> vector(size_t i, int shift) {
-            return VectorPointerHelper<V, UnalignedFlag>(&entries()[i * V::Size + shift]);
+        template <typename ShiftT, typename Flags = decltype(Unaligned)>
+        Vc_ALWAYS_INLINE Vc_PURE typename std::enable_if<
+            std::is_convertible<ShiftT, int>::value,
+            MemoryVector<V, decltype(std::declval<Flags>() | Unaligned)>>::type &
+        vector(size_t i, ShiftT shift, Flags = Flags())
+        {
+            return *new (&entries()[i * V::Size + shift])
+                MemoryVector<V, decltype(std::declval<Flags>() | Unaligned)>;
         }
         /// Const overload of the above function.
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, UnalignedFlag> vector(size_t i, int shift) const {
-            return VectorPointerHelperConst<V, UnalignedFlag>(&entries()[i * V::Size + shift]);
+        template <typename ShiftT, typename Flags = decltype(Unaligned)>
+        Vc_ALWAYS_INLINE Vc_PURE typename std::enable_if<
+            std::is_convertible<ShiftT, int>::value,
+            MemoryVector<const V, decltype(std::declval<Flags>() | Unaligned)>>::type &
+        vector(size_t i, ShiftT shift, Flags = Flags()) const
+        {
+            return *new (const_cast<EntryType *>(&entries()[i * V::Size + shift]))
+                MemoryVector<const V, decltype(std::declval<Flags>() | Unaligned)>;
         }
 
         /**
@@ -411,12 +531,14 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          *
          * This function is simply a shorthand for vector(0).
          */
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, AlignedFlag> firstVector() {
-            return VectorPointerHelper<V, AlignedFlag>(entries());
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<V, Flags> &firstVector(Flags = Flags()) {
+            return *new(entries()) MemoryVector<V, Flags>;
         }
         /// Const overload of the above function.
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, AlignedFlag> firstVector() const {
-            return VectorPointerHelperConst<V, AlignedFlag>(entries());
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<const V, Flags> &firstVector(Flags = Flags()) const {
+            return *new(const_cast<EntryType *>(entries())) MemoryVector<const V, Flags>;
         }
 
         /**
@@ -424,12 +546,14 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
          *
          * This function is simply a shorthand for vector(vectorsCount() - 1).
          */
-        Vc_ALWAYS_INLINE Vc_PURE VectorPointerHelper<V, AlignedFlag> lastVector() {
-            return VectorPointerHelper<V, AlignedFlag>(&entries()[vectorsCount() * V::Size - V::Size]);
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<V, Flags> &lastVector(Flags = Flags()) {
+            return *new(&entries()[vectorsCount() * V::Size - V::Size]) MemoryVector<V, Flags>;
         }
         /// Const overload of the above function.
-        Vc_ALWAYS_INLINE Vc_PURE const VectorPointerHelperConst<V, AlignedFlag> lastVector() const {
-            return VectorPointerHelperConst<V, AlignedFlag>(&entries()[vectorsCount() * V::Size - V::Size]);
+        template<typename Flags = AlignedTag>
+        Vc_ALWAYS_INLINE Vc_PURE MemoryVector<const V, Flags> &lastVector(Flags = Flags()) const {
+            return *new(const_cast<EntryType *>(&entries()[vectorsCount() * V::Size - V::Size])) MemoryVector<const V, Flags>;
         }
 
         Vc_ALWAYS_INLINE Vc_PURE V gather(const unsigned char  *indexes) const { return V(entries(), indexes); }
@@ -437,6 +561,9 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
         Vc_ALWAYS_INLINE Vc_PURE V gather(const unsigned int   *indexes) const { return V(entries(), indexes); }
         Vc_ALWAYS_INLINE Vc_PURE V gather(const unsigned long  *indexes) const { return V(entries(), indexes); }
 
+        /**
+         * Zero the whole memory area.
+         */
         Vc_ALWAYS_INLINE void setZero() {
             V zero(Vc::Zero);
             for (size_t i = 0; i < vectorsCount(); ++i) {
@@ -444,6 +571,19 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
         }
 
+        /**
+         * Assign a value to all vectors in the array.
+         */
+        template<typename U>
+        Vc_ALWAYS_INLINE Parent &operator=(U &&x) {
+            for (size_t i = 0; i < vectorsCount(); ++i) {
+                vector(i) = std::forward<U>(x);
+            }
+        }
+
+        /**
+         * (Inefficient) shorthand to add up two arrays.
+         */
         template<typename P2, typename RM>
         inline Parent &operator+=(const MemoryBase<V, P2, Dimension, RM> &rhs) {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -452,6 +592,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to subtract two arrays.
+         */
         template<typename P2, typename RM>
         inline Parent &operator-=(const MemoryBase<V, P2, Dimension, RM> &rhs) {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -460,6 +604,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to multiply two arrays.
+         */
         template<typename P2, typename RM>
         inline Parent &operator*=(const MemoryBase<V, P2, Dimension, RM> &rhs) {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -468,6 +616,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to divide two arrays.
+         */
         template<typename P2, typename RM>
         inline Parent &operator/=(const MemoryBase<V, P2, Dimension, RM> &rhs) {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -476,6 +628,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to add a value to an array.
+         */
         inline Parent &operator+=(EntryType rhs) {
             V v(rhs);
             for (size_t i = 0; i < vectorsCount(); ++i) {
@@ -483,6 +639,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to subtract a value from an array.
+         */
         inline Parent &operator-=(EntryType rhs) {
             V v(rhs);
             for (size_t i = 0; i < vectorsCount(); ++i) {
@@ -490,6 +650,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to multiply a value to an array.
+         */
         inline Parent &operator*=(EntryType rhs) {
             V v(rhs);
             for (size_t i = 0; i < vectorsCount(); ++i) {
@@ -497,6 +661,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand to divide an array with a value.
+         */
         inline Parent &operator/=(EntryType rhs) {
             V v(rhs);
             for (size_t i = 0; i < vectorsCount(); ++i) {
@@ -504,6 +672,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return static_cast<Parent &>(*this);
         }
+
+        /**
+         * (Inefficient) shorthand compare equality of two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator==(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -514,6 +686,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return true;
         }
+
+        /**
+         * (Inefficient) shorthand compare two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator!=(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -524,6 +700,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return true;
         }
+
+        /**
+         * (Inefficient) shorthand compare two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator<(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -534,6 +714,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return true;
         }
+
+        /**
+         * (Inefficient) shorthand compare two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator<=(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -544,6 +728,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return true;
         }
+
+        /**
+         * (Inefficient) shorthand compare two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator>(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -554,6 +742,10 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
             }
             return true;
         }
+
+        /**
+         * (Inefficient) shorthand compare two arrays.
+         */
         template<typename P2, typename RM>
         inline bool operator>=(const MemoryBase<V, P2, Dimension, RM> &rhs) const {
             assert(vectorsCount() == rhs.vectorsCount());
@@ -566,7 +758,7 @@ template<typename V, typename Parent, int Dimension, typename RowMemory> class M
         }
 };
 
-namespace Internal
+namespace Detail
 {
 template <typename V,
           typename ParentL,
@@ -593,11 +785,11 @@ inline void copyVectors(MemoryBase<V, ParentL, Dimension, RowMemoryL> &dst,
         dst.vector(i) = src.vector(i);
     }
 }
-} // namespace Internal
+} // namespace Detail
 
-} // namespace Vc
-/*OUTER_NAMESPACE_END*/
+}  // namespace Common
+}  // namespace Vc
 
-#include "undomacros.h"
+#endif // VC_COMMON_MEMORYBASE_H_
 
-#endif // VC_COMMON_MEMORYBASE_H
+// vim: foldmethod=marker
