@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2010-2011 Matthias Kretz <kretz@kde.org>
+    Copyright (C) 2010-2015 Matthias Kretz <kretz@kde.org>
 
     Permission to use, copy, modify, and distribute this software
     and its documentation for any purpose and without fee is hereby
@@ -31,8 +31,8 @@
 
 using Vc::float_v;
 using Vc::float_m;
-using Vc::uint_v;
-using Vc::uint_m;
+using uint_v = Vc::SimdArray<unsigned int, float_v::size()>;
+using uint_m = uint_v::mask_type;
 
 template<MandelImpl Impl>
 Mandel<Impl>::Mandel(QObject *_parent)
@@ -88,16 +88,16 @@ void MandelBase::run()
 
         // benchmark the number of cycles it takes
         TimeStampCounter timer;
-        timer.Start();
+        timer.start();
 
         // calculate the mandelbrot set/image
         mandelMe(image, x, y, scale, 255);
 
-        timer.Stop();
+        timer.stop();
 
         // if no new set was requested in the meantime - return the finished image
         if (!m_restart) {
-            emit ready(image, timer.Cycles());
+            emit ready(image, timer.cycles());
         }
 
         // wait for more work
@@ -166,21 +166,21 @@ template<> void Mandel<VcImpl>::mandelMe(QImage &image, float x0,
     const unsigned int width = image.width();
     const float_v colorScale = 0xff / static_cast<float>(maxIt);
     for (unsigned int y = 0; y < height; ++y) {
-        unsigned int *VC_RESTRICT line = reinterpret_cast<unsigned int *>(image.scanLine(y));
+        unsigned int *Vc_RESTRICT line = reinterpret_cast<unsigned int *>(image.scanLine(y));
         const float_v c_imag = y0 + y * scale;
         uint_m toStore;
         for (uint_v x = uint_v::IndexesFromZero(); !(toStore = x < width).isEmpty();
                 x += float_v::Size) {
-            const float_v c_real = x0 + x * scale;
+            const float_v c_real = x0 + simd_cast<float_v>(x) * scale;
             Z z(c_real, c_imag);
-            float_v n = 0.f;
+            float_v n = float_v::Zero();
             float_m inside = z.norm() < S;
             while (!(inside && n < maxIt).isEmpty()) {
                 z = P(z, c_real, c_imag);
                 ++n(inside);
                 inside = z.norm() < S;
             }
-            uint_v colorValue = static_cast<uint_v>((maxIt - n) * colorScale) * 0x10101;
+            uint_v colorValue = simd_cast<uint_v>((maxIt - n) * colorScale) * 0x10101;
             if (toStore.isFull()) {
                 colorValue.store(line, Vc::Unaligned);
                 line += uint_v::Size;
@@ -203,7 +203,7 @@ template<> void Mandel<ScalarImpl>::mandelMe(QImage &image, float x0,
     const int width = image.width();
     const float colorScale = 0xff / static_cast<float>(maxIt);
     for (int y = 0; y < height; ++y) {
-        unsigned int *VC_RESTRICT line = reinterpret_cast<unsigned int *>(image.scanLine(y));
+        unsigned int *Vc_RESTRICT line = reinterpret_cast<unsigned int *>(image.scanLine(y));
         const float c_imag = y0 + y * scale;
         for (int x = 0; x < width; ++x) {
             const float c_real = x0 + x * scale;
