@@ -1,27 +1,35 @@
-/*  This file is part of the Vc library.
+/*  This file is part of the Vc library. {{{
+Copyright © 2009-2015 Matthias Kretz <kretz@kde.org>
 
-    Copyright (C) 2009-2012 Matthias Kretz <kretz@kde.org>
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+    * Redistributions of source code must retain the above copyright
+      notice, this list of conditions and the following disclaimer.
+    * Redistributions in binary form must reproduce the above copyright
+      notice, this list of conditions and the following disclaimer in the
+      documentation and/or other materials provided with the distribution.
+    * Neither the names of contributing organizations nor the
+      names of its contributors may be used to endorse or promote products
+      derived from this software without specific prior written permission.
 
-    Vc is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as
-    published by the Free Software Foundation, either version 3 of
-    the License, or (at your option) any later version.
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-    Vc is distributed in the hope that it will be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+}}}*/
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Vc.  If not, see <http://www.gnu.org/licenses/>.
+#ifndef VC_CPUID_H_
+#define VC_CPUID_H_
 
-*/
-
-#ifndef CPUID_H
-#define CPUID_H
-
-/*OUTER_NAMESPACE_BEGIN*/
-namespace Vc
+#include "common/macros.h"
+namespace Vc_VERSIONED_NAMESPACE
 {
 
 /**
@@ -115,6 +123,36 @@ class CpuId
         static inline bool hasOsxsave() { return (s_processorFeaturesC & (1 << 27)) != 0; }
         //! Return whether the CPU supports AVX.
         static inline bool hasAvx  () { return (s_processorFeaturesC & (1 << 28)) != 0; }
+        //! Return whether the CPU supports BMI1.
+        static inline bool hasBmi1 () { return (s_processorFeatures7B& (1 << 3)) != 0; }
+        //! Return whether the CPU supports transactional synchronization extensions.
+        static inline bool hasHle  () { return (s_processorFeatures7B& (1 << 4)) != 0; }
+        //! Return whether the CPU supports AVX2.
+        static inline bool hasAvx2 () { return (s_processorFeatures7B& (1 << 5)) != 0; }
+        //! Return whether the CPU supports BMI2.
+        static inline bool hasBmi2 () { return (s_processorFeatures7B& (1 << 8)) != 0; }
+        //! Return whether the CPU supports transactional synchronization extensions.
+        static inline bool hasRtm  () { return (s_processorFeatures7B& (1 << 11)) != 0; }
+
+        //! Return whether the CPU supports AVX512f.
+        static inline bool hasAvx512f   () { return (s_processorFeatures7B & (1u << 16)) != 0; }
+        //! Return whether the CPU supports AVX512dq.
+        static inline bool hasAvx512dq  () { return (s_processorFeatures7B & (1u << 17)) != 0; }
+        //! Return whether the CPU supports AVX512ifma.
+        static inline bool hasAvx512ifma() { return (s_processorFeatures7B & (1u << 21)) != 0; }
+        //! Return whether the CPU supports AVX512pf.
+        static inline bool hasAvx512pf  () { return (s_processorFeatures7B & (1u << 26)) != 0; }
+        //! Return whether the CPU supports AVX512er.
+        static inline bool hasAvx512er  () { return (s_processorFeatures7B & (1u << 27)) != 0; }
+        //! Return whether the CPU supports AVX512cd.
+        static inline bool hasAvx512cd  () { return (s_processorFeatures7B & (1u << 28)) != 0; }
+        //! Return whether the CPU supports AVX512bw.
+        static inline bool hasAvx512bw  () { return (s_processorFeatures7B & (1u << 30)) != 0; }
+        //! Return whether the CPU supports AVX512vl.
+        static inline bool hasAvx512vl  () { return (s_processorFeatures7B & (1u << 31)) != 0; }
+        //! Return whether the CPU supports AVX512vbmi.
+        static inline bool hasAvx512vbmi() { return (s_processorFeatures7C & (1u <<  1)) != 0; }
+
         //! Return whether the CPU supports 16-bit floating-point conversion instructions.
         static inline bool hasF16c () { return (s_processorFeaturesC & (1 << 29)) != 0; }
         //! Return whether the CPU supports the RDRAND instruction.
@@ -187,6 +225,8 @@ class CpuId
         static uint   s_logicalProcessors;
         static uint   s_processorFeaturesC;
         static uint   s_processorFeaturesD;
+        static uint   s_processorFeatures7B;
+        static uint   s_processorFeatures7C;
         static uint   s_processorFeatures8C;
         static uint   s_processorFeatures8D;
         static uint   s_L1Instruction;
@@ -208,7 +248,22 @@ class CpuId
         static ProcessorType s_processorType;
         static bool   s_noL2orL3;
 };
-} // namespace Vc
-/*OUTER_NAMESPACE_END*/
 
-#endif // CPUID_H
+#ifndef Vc_COMPILE_LIB
+namespace detail
+{
+template <int = 0> struct RunCpuIdInit {
+    RunCpuIdInit() { CpuId::init(); }
+    static RunCpuIdInit tmp;
+};
+template <int N> RunCpuIdInit<N> RunCpuIdInit<N>::tmp;
+namespace
+{
+static auto ctor = RunCpuIdInit<>::tmp;
+}  // unnamed namespace
+}  // namespace detail
+#endif
+
+}
+
+#endif // VC_CPUID_H_
