@@ -1,39 +1,50 @@
-/*  This file is part of the Vc library.
+/*  This file is part of the Vc library. {{{
+Copyright © 2010-2015 Matthias Kretz <kretz@kde.org>
 
-    Copyright (C) 2010-2012 Matthias Kretz <kretz@kde.org>
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+    * Redistributions of source code must retain the above copyright
+      notice, this list of conditions and the following disclaimer.
+    * Redistributions in binary form must reproduce the above copyright
+      notice, this list of conditions and the following disclaimer in the
+      documentation and/or other materials provided with the distribution.
+    * Neither the names of contributing organizations nor the
+      names of its contributors may be used to endorse or promote products
+      derived from this software without specific prior written permission.
 
-    Vc is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as
-    published by the Free Software Foundation, either version 3 of
-    the License, or (at your option) any later version.
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-    Vc is distributed in the hope that it will be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+}}}*/
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Vc.  If not, see <http://www.gnu.org/licenses/>.
+#ifndef VC_COMMON_SUPPORT_H_
+#define VC_COMMON_SUPPORT_H_
 
-*/
-
-#ifndef VC_COMMON_SUPPORT_H
-#define VC_COMMON_SUPPORT_H
-
-#ifndef VC_GLOBAL_H
+#ifndef VC_GLOBAL_H_
 #error "Vc/global.h must be included first!"
 #endif
 
+#if defined __x86_64__ || defined __amd64__ || defined __amd64 || defined __x86_64 ||    \
+    defined _M_AMD64 || defined __i386__
 #include <Vc/cpuid.h>
-
-#if defined(VC_GCC) && VC_GCC >= 0x40400
-#define VC_TARGET_NO_SIMD __attribute__((target("no-sse2,no-avx")))
-#else
-#define VC_TARGET_NO_SIMD
 #endif
 
-/*OUTER_NAMESPACE_BEGIN*/
-namespace Vc
+#if defined(Vc_GCC) && Vc_GCC >= 0x40400 && defined __SSE__
+#define Vc_TARGET_NO_SIMD __attribute__((target("no-sse2,no-avx")))
+#else
+#define Vc_TARGET_NO_SIMD
+#endif
+
+#include "common/macros.h"
+namespace Vc_VERSIONED_NAMESPACE
 {
 
 /**
@@ -47,7 +58,7 @@ namespace Vc
  *
  * \return A combination of flags from Vc::ExtraInstructions that the current CPU supports.
  */
-VC_TARGET_NO_SIMD
+Vc_TARGET_NO_SIMD
 unsigned int extraInstructionsSupported();
 
 /**
@@ -61,7 +72,7 @@ unsigned int extraInstructionsSupported();
  *
  * \param impl The SIMD target to test for.
  */
-VC_TARGET_NO_SIMD
+Vc_TARGET_NO_SIMD
 bool isImplementationSupported(Vc::Implementation impl);
 
 /**
@@ -84,11 +95,11 @@ bool isImplementationSupported(Vc::Implementation impl);
  * \tparam Impl The SIMD target to test for.
  */
 template<typename Impl>
-VC_TARGET_NO_SIMD
+Vc_TARGET_NO_SIMD
 static inline bool isImplementationSupported()
 {
-    return isImplementationSupported(static_cast<Vc::Implementation>(Impl::Implementation)) &&
-        (extraInstructionsSupported() & Impl::ExtraInstructions) == Impl::ExtraInstructions;
+    return isImplementationSupported(Impl::current()) &&
+           Impl::runs_on(extraInstructionsSupported());
 }
 
 /**
@@ -99,10 +110,10 @@ static inline bool isImplementationSupported()
  *
  * \return The enum value for the best implementation.
  */
-VC_TARGET_NO_SIMD
+Vc_TARGET_NO_SIMD
 Vc::Implementation bestImplementationSupported();
 
-#ifndef VC_COMPILE_LIB
+#ifndef Vc_COMPILE_LIB
 /**
  * \ingroup Utilities
  * \headerfile support.h <Vc/support.h>
@@ -131,7 +142,7 @@ Vc::Implementation bestImplementationSupported();
  *                 instructions.
  * \return \c false otherwise
  */
-VC_TARGET_NO_SIMD
+Vc_TARGET_NO_SIMD
 #ifndef DOXYGEN
 static
 #endif
@@ -139,12 +150,11 @@ inline bool currentImplementationSupported()
 {
     return isImplementationSupported<Vc::CurrentImplementation>();
 }
-#endif // VC_COMPILE_LIB
+#endif // Vc_COMPILE_LIB
 //@}
 
-} // namespace Vc
-/*OUTER_NAMESPACE_END*/
+}
 
-#undef VC_TARGET_NO_SIMD
+#undef Vc_TARGET_NO_SIMD
 
-#endif // VC_COMMON_SUPPORT_H
+#endif // VC_COMMON_SUPPORT_H_
