@@ -2,7 +2,6 @@
 
 cd "`dirname "$0"`/doc"
 
-rm -rf html latex man
+rm -rf html
 doxygen
-cd latex
-make
+rm -f html/*.qhp
