@@ -3,7 +3,7 @@
 cd "`dirname "$0"`"
 
 # Read version number
-eval `awk '/VC_VERSION_NUMBER 0x[0-9]+/ { h=$3 }
+eval `awk '/Vc_VERSION_NUMBER 0x[0-9]+/ { h=$3 }
 END {
 major=strtonum(substr(h, 1, 4))
 minor=strtonum("0x" substr(h, 5, 2))
@@ -24,15 +24,13 @@ sed -i \
 	-e "s/^HTML_TIMESTAMP         = YES/HTML_TIMESTAMP         = NO/" \
 	doc/Doxyfile
 sed -i \
-	-e "s/VC_VERSION_STRING \".*\"\$/VC_VERSION_STRING \"$versionString\"/" \
-	-e "s/VC_VERSION_NUMBER 0x.*\$/VC_VERSION_NUMBER $versionNumber/" \
+	-e "s/Vc_VERSION_STRING \".*\"\$/Vc_VERSION_STRING \"$versionString\"/" \
+	-e "s/Vc_VERSION_NUMBER 0x.*\$/Vc_VERSION_NUMBER $versionNumber/" \
 	include/Vc/version.h
 cat include/Vc/version.h
 
 # Don't build tests with make all
-sed -i \
-  -e 's/add_custom_target(build_tests ALL VERBATIM)/add_custom_target(build_tests VERBATIM)/' \
-  CMakeLists.txt
+sed -i -e 's/#Release# //' CMakeLists.txt
 git commit CMakeLists.txt doc/Doxyfile include/Vc/version.h -s -F- <<EOF
 release: version $versionString
 
@@ -52,6 +50,9 @@ mv doc/html/*.qch "../Vc-${versionString}.qch"
 mv doc/html "Vc-docs-$versionString" && tar -czf "../Vc-docs-$versionString".tar.gz "Vc-docs-$versionString"
 rm -rf "Vc-docs-$versionString"
 
+# Get back to the state before the tag and fix up the version numbers afterwards
+git revert -n HEAD
+
 # Update the version number of the after-release code
 versionString="$versionString-dev"
 versionNumber=`echo $versionNumber | awk '{ printf "0x%06x", (strtonum($0) + 1) }'`
@@ -61,13 +62,9 @@ sed -i \
 	-e "s/^HTML_TIMESTAMP         = YES/HTML_TIMESTAMP         = NO/" \
 	doc/Doxyfile
 sed -i \
-	-e "s/VC_VERSION_STRING \".*\"\$/VC_VERSION_STRING \"$versionString\"/" \
-	-e "s/VC_VERSION_NUMBER 0x.*\$/VC_VERSION_NUMBER $versionNumber/" \
+	-e "s/Vc_VERSION_STRING \".*\"\$/Vc_VERSION_STRING \"$versionString\"/" \
+	-e "s/Vc_VERSION_NUMBER 0x.*\$/Vc_VERSION_NUMBER $versionNumber/" \
 	include/Vc/version.h
-# Revert the build_tests change
-sed -i \
-  -e 's/add_custom_target(build_tests VERBATIM)/add_custom_target(build_tests ALL VERBATIM)/' \
-  CMakeLists.txt
 git commit CMakeLists.txt doc/Doxyfile include/Vc/version.h -s -F- <<EOF
 after release: version $versionString
 
