@@ -727,7 +727,7 @@ template <typename T> struct DetermineGatherMask
 template <typename T> struct VectorTraits
 {
     typedef typename VectorTypeHelper<T>::Type VectorType;
-    Vc_ALIGNED_TYPEDEF(sizeof(T), T, EntryType);
+    using EntryType = typename Common::ensure_alignment_equals_sizeof<T>::type;
     static constexpr size_t Size = sizeof(VectorType) / sizeof(EntryType);
     enum Constants { HasVectorDivision = !std::is_integral<T>::value };
     typedef Mask<T> MaskType;
